<?php namespace Helper\MetadataGenerator\Metadata\OpenGraph;
    
    use Helper\MetadataGenerator\Metadata\Utils\Utils;

    class OpenGraphMetadata {
        /**
        * @var string
        */
        public $admins = "";
        /**
        * @var string
        */
        public $app_id = "";
        /**
        * @var string
        */
        public $type = "";
        /**
        * @var string
        */
        public $title = "";
        /**
        * @var string
        */
        public $description = "";
        /**
        * @var string
        */
        public $imageSrc = "";
        /**
        * @var string
        */
        public $link = "";
        /**
        * @var struct
        */
        public $ogImage = null;
        /**
        * @var struct
        */
        public $ogVideo = null;
        /**
        * @var struct
        */
        public $ogAudio = null;
         /**
        * @var struct
        */
        public $locale = "";
         /**
        * @var struct
        */
        public $l10n = array();
         /**
        * @var struct
        */
        public $siteName = "";
        
        

        public $prettyPrint = false;     

        public function __construct ($format = false){
            $this->prettyPrint = $format;
        }
        /**
        * generate string for Admins
        *
        * @return string
        */
        private function printAdmins(){
            if($this->admins != "")
                return Utils::print_metaPC("fb:admins",$this->admins);
            return "";
        }
        /**
        * generate string for appId
        *
        * @return string
        */
        private function printAppId(){
            if($this->app_id != "")
                return Utils::print_metaPC("fb:page_id",$this->app_id);
            return "";
        }
        /**
        * generate string for pages
        *
        * @return string
        */
        private function printPages(){
            if($this->app_id != "")
                return Utils::print_metaPC("fb:pages",$this->app_id);
            return "";
        }
        /**
        * generate string for title
        *
        * @return string
        */
        private function printTitle(){
            if($this->title != ""){
                $title =  htmlentities($this->title,ENT_QUOTES,"UTF-8");
                return Utils::print_metaPC("og:title",$title);
            }
            return "";
            
        }
        /**
        * generate string for description
        *
        * @return string
        */
        private function printDescription(){      
            if($this->description != ""){
                $description = htmlentities($this->description,ENT_QUOTES,"UTF-8");
                return  Utils::print_metaPC("og:description",$description);
            }
            return "";
            
        }
        /**
        * generate string for link
        *
        * @return string
        */
        private function printLink(){
            if($this->link != ""){
                return Utils::print_metaPC("og:url",$this->link);
            }
            return "";
            
        }
        /**
        * generate string for locale and alternate locales
        *
        * @return string
        */
        private function printLocale(){
            $locale = "";
            if($this->locale != ""){
                $locale .= Utils::print_metaPC("og:locale",$this->locale);
                if(is_array($this->l10n) && count($this->l10n) > 0){
                    foreach($this->l10n as $lang){
                        if($lang != $this->locale)
                            $locale .= ($this->prettyPrint ? "\n" : "").Utils::print_metaPC("og:locale:alternate",$lang);
                    }
                }
            }
            return $locale;
            
        }
         /**
        * generate string for sitename
        *
        * @return string
        */
        private function printSiteName(){
            if($this->siteName != ""){
                return Utils::print_metaPC("og:site_name",htmlentities($this->siteName,ENT_QUOTES,"UTF-8") );
            }
            return "";
            
        }
         /**
        * generate string for type
        *
        * @return string
        */
        private function printType(){
            if($this->type != ""){
                return Utils::print_metaPC("og:type",$this->type);
            }
            return "";
            
        }
        /**
        * generate string for image
        *
        * @return string
        */
        private function printImage(){
            if($this->ogImage == null){

                return $this->imageSrc != "" ? Utils::print_metaPC("og:image",$this->imageSrc)  : "";
            }else{
                //build ogImageWithalloptions
                $image =  isset($this->ogImage["url"]) ? Utils::print_metaPC("og:image",$this->ogImage["url"]) : "";
                if($image != ""){
                    $url = isset($this->ogImage["url"]) ? Utils::print_metaPC("og:image:url",$this->ogImage["url"]) : "";
                    $secure_url = isset($this->ogImage["secure_url"]) ? Utils::print_metaPC("og:image:secure_url",$this->ogImage["secure_url"]) : "";
                    $type = isset($this->ogImage["type"]) ? Utils::print_metaPC("og:image:type",$this->ogImage["type"]) : "";
                    $width = isset($this->ogImage["width"]) ? Utils::print_metaPC("og:image:width",$this->ogImage["width"]) : "";
                    $height = isset($this->ogImage["height"]) ? Utils::print_metaPC("og:image:height",$this->ogImage["height"]) : "";
                    $alt = isset($this->ogImage["alt"]) ? Utils::print_metaPC("og:image:alt",$this->ogImage["alt"]) : "";


                    return $image.($this->prettyPrint ? "\n" : "").
                           $url.($this->prettyPrint ? "\n" : "").
                           $secure_url.($this->prettyPrint ? "\n" : "").
                           $type.($this->prettyPrint ? "\n" : "").
                           $width.($this->prettyPrint ? "\n" : "").
                           $height.($this->prettyPrint ? "\n" : "").
                           $alt;
                }
                
                
            }
            return "";
            
        }
        /**
        * generate string for video
        *
        * @return string
        */
        private function printVideo(){
            if($this->ogVideo != null){
               $video = isset($this->ogVideo["url"]) ? Utils::print_metaPC("og:video",$this->ogVideo["url"]) : "";
               if($video != ""){
                    $url = isset($this->ogVideo["url"]) ? Utils::print_metaPC("og:video:url",$this->ogVideo["url"]) : "";
                    $secure_url = isset($this->ogVideo["secure_url"]) ? Utils::print_metaPC("og:video:secure_url",$this->ogVideo["secure_url"]) : "";
                    $type = isset($this->ogVideo["type"]) ? Utils::print_metaPC("og:video:type",$this->ogVideo["type"]) : "";
                    $width = isset($this->ogVideo["width"]) ? Utils::print_metaPC("og:video:width",$this->ogVideo["width"]) : "";
                    $height = isset($this->ogVideo["height"]) ? Utils::print_metaPC("og:video:height",$this->ogVideo["height"]) : "";
                    $duration = isset($this->ogVideo["duration"]) ? Utils::print_metaPC("og:video:duration",$this->ogVideo["duration"]) : "";
    
                    return $video.($this->prettyPrint ? "\n" : "").
                        $url.($this->prettyPrint ? "\n" : "").
                        $secure_url.($this->prettyPrint ? "\n" : "").
                        $type.($this->prettyPrint ? "\n" : "").
                        $width.($this->prettyPrint ? "\n" : "").
                        $height.($this->prettyPrint ? "\n" : "").
                        $duration;
               }
               
            }

            return "";
        }
        /**
        * generate string for audio
        *
        * @return string
        */
        private function printAudio(){
            if($this->ogAudio != null){
                $audio = isset($this->ogAudio["url"]) ? Utils::print_metaPC("og:audio",$this->ogAudio["url"]) : "";
                if($audio != ""){
                     $url = isset($this->ogAudio["url"]) ? Utils::print_metaPC("og:audio:url",$this->ogAudio["url"]) : "";
                     $secure_url = isset($this->ogAudio["secure_url"]) ? Utils::print_metaPC("og:audio:secure_url",$this->ogAudio["secure_url"]) : "";
                     $type = isset($this->ogAudio["type"]) ? Utils::print_metaPC("og:audio:type",$this->ogAudio["type"]) : "";
     
                     return $audio.($this->prettyPrint ? "\n" : "").
                         $url.($this->prettyPrint ? "\n" : "").
                         $secure_url.($this->prettyPrint ? "\n" : "").
                         $type; 
                }
                
             }
 
             return "";
        }
        /**
        * generate string for all openGraphMetadata
        *
        * @return string
        */
        public function toString(){
            return $this->printLink(). ($this->prettyPrint ? "\n" : "").
                   $this->printAdmins(). ($this->prettyPrint ? "\n" : "").
                   //$this->printAppId(). ($this->prettyPrint ? "\n" : "").
                   $this->printPages(). ($this->prettyPrint ? "\n" : "").
                   $this->printTitle(). ($this->prettyPrint ? "\n" : "").
                   $this->printDescription(). ($this->prettyPrint ? "\n" : "").
                   $this->printLocale(). ($this->prettyPrint ? "\n" : "").
                   $this->printSiteName(). ($this->prettyPrint ? "\n" : "").
                   $this->printImage(). ($this->prettyPrint ? "\n" : "").
                   $this->printVideo(). ($this->prettyPrint ? "\n" : "").
                   $this->printAudio(). ($this->prettyPrint ? "\n" : "").
                   $this->printType();
        }
    }