<?php namespace Helper\MetadataGenerator\StructuredData\Article;

use Helper\MetadataGenerator\StructuredData\JsonLDGenerator;

class ArticleJsonLD extends JsonLDGenerator{
   
    public $image = "";
    public $pubDate = "";
    public $modDate = "";
    public $authors = array();
    public $publisher = "";
    public $favicon = "";
    public $sitename = "";

    public $formated = 0;
    public function __construct($format = false){
      parent::__construct();
      $this->type = "NewsArticle";
      $this->formated = $format ? 128 : 0;
    }

    private function printImage(){
        if($this->image != ""){
          return array( $this->image );
        }
        return array();
    }

    private function printPublisher(){
      if(is_array($this->favicon) && array_key_exists("favicon",$this->favicon)){
        $this->favicon = $this->favicon["favicon"];
      }
      return array(
          "@type" => "Organization", //Organization or Person
          "name" => $this->publisher, 
          "logo" => array(
            "@type" => "ImageObject", // imageObject
            "url" => $this->favicon //
          )
        );
    }

    private function printAuthor($author){
      if( is_array($author) ){
        $name = isset($author["name"]) ? htmlentities($author["name"],ENT_QUOTES,"UTF-8") : "";
      }else{
        $name = $author;
      }

      return array(
          "@type" => "Person",
          "name" => $name
        );
    }
    private function printAuthors(){
       $arr = array();
       if(count($this->authors) > 0){
         foreach($this->authors as $author){
            array_push($arr,$this->printAuthor($author));
         }
       }
       return $arr;
    }

    public function toString(){
        $data = array();
        $data["@context"] = $this->context;
        $data["@type"]  = $this->type;
        $data["mainEntityOfPage"] = array(
            "@type" => $this->ent_type,
            "@id" => $this->ent_id
        );
        $data["headline"] = htmlentities($this->title,ENT_QUOTES,"UTF-8");
        $data["image"] = $this->printImage();
        $data["datePublished"] = $this->pubDate;
        $data["dateModified"] = $this->modDate;
        $data["description"] = htmlentities($this->description,ENT_QUOTES,"UTF-8");
        $data["publisher"] = $this->printPublisher();
        $data["author"] = $this->printAuthors();

        return json_encode($data,$this->formated);
    }
}


/*<script type="application/ld+json">
{
  "@context": "http://schema.org",
  "@type": "NewsArticle",
  "mainEntityOfPage": {
    "@type": "WebPage", //webPage
    "@id": "https://google.com/article" //canonical
  },
  "headline": "Article headline", // title
  "image": [
    "https://example.com/photos/1x1/photo.jpg", //imgSrc
    "https://example.com/photos/4x3/photo.jpg",
    "https://example.com/photos/16x9/photo.jpg"
   ],
  "datePublished": "2015-02-05T08:00:00+08:00", //pubDate
  "dateModified": "2015-02-05T09:20:00+08:00", //modDate
  "author": {
    "@type": "Person",
    "name": "John Doe"
  },  //could be an array of stuff
   "publisher": {
    "@type": "Organization", //Organization or Person
    "name": "Google", // sitename or company name
    "logo": {
      "@type": "ImageObject", // imageObject
      "url": "https://google.com/logo.jpg" //
    }
  },
  "description": "A most wonderful article" //description
}
</script>
*/