<?php namespace Helper\MetadataGenerator\StructuredData\Event;

use Helper\MetadataGenerator\StructuredData\JsonLDGenerator;
use Helper\MetadataGenerator\StructuredData\StockManagement;
use Helper\MetadataGenerator\StructuredData\Rating\AggregateRatingsJsonLD;
use Helper\MetadataGenerator\StructuredData\Review\ReviewJsonLD;

class EventJsonLD extends JsonLDGenerator{

    public $eventName = "";

    public $startDate = "";

    public $address = array();

    public $placeName  ="";
    
    public $image = array();
    
    public $description = "";

    public $endDate = "";

    public $performer = "";

    public $performerType = "";

    public $offers = array();
   
    public $formated = 0;

    public $ratings = array();

    public $reviews = array();
    

    public function __construct($format = false){
      parent::__construct();
      $this->type = "Event";
      $this->formated = $format ? 128 : 0;
    }

    private function printLocation(){
        $data = array();
        if($this->placeName != "" && count($this->address) > 0){
            $data["@type"] = "Place";
            $data["name"] = $this->placeName;
            $data["address"] = array();
            $data["address"]["streetAddress"] = isset($this->address["streetAddress"]) ? $this->address["streetAddress"] : "";
            $data["address"]["addressLocality"] = isset($this->address["city"]) ? $this->address["city"] : "";
            $data["address"]["addressRegion"] = isset($this->address["region"]) ? $this->address["region"] : "";
            $data["address"]["postalCode"] = isset($this->address["zipcode"]) ? $this->address["zipcode"] : "";
            $data["address"]["addressCountry"] = isset($this->address["country"]) ? $this->address["country"] : "";
        }

        return $data;
    }

    private function printOffers(){
        $data = array();
        if(isset($this->offers) && count($this->offers) > 0){
           foreach($this->offers as $offer){
               $off = array();
               $off["@type"] =  "Offer";
               $off["url"] =  isset($offer["url"]) ? $offer["url"]  : "";
               $off["price"] =  isset($offer["price"]) ? $offer["price"]  : "";
               $off["priceCurrency"] =  isset($offer["currency"]) ? $offer["currency"] : "";
               $off["availability"] =  isset($offer["availability"]) && StockManagement::isValidValue($offer["availability"],true) ? $offer["availability"] : StockManagement::InStock;
               $off["validFrom"] =  isset($offer["startDate"]) ? $offer["startDate"] : "" ;
               $off["validThrough"] = isset($offer["endDate"]) ? $offer["endDate"] : "" ;

               array_push($data,$off);
           }
        }

        return $data;
    }

    private function printRatings(){
        $ratings = new AggregateRatingsJsonLD();
        $ratings->value = isset($this->ratings["value"]) ? $this->ratings["value"] : 0;
        $ratings->maxValue  = isset($this->ratings["maxValue"]) ? $this->ratings["maxValue"] : 0;
        $ratings->minValue  = isset($this->ratings["minValue"]) ? $this->ratings["minValue"] : 0;
        $ratings->totalRatings  = isset($this->ratings["totalRatings"]) ? $this->ratings["totalRatings"] : 0;

        return $ratings->toArray();

    }

    private function printReviews(){
        $data = array();
        if(isset($this->reviews)){
            foreach($this->reviews as $rev){
                $review = new ReviewJsonLD();
                $review->url = isset($rev["url"]) ? $rev["url"] : "";
                $review->authors = isset($rev["authors"]) ? $rev["authors"] : array();
                $review->publisher = isset($rev["publisher"]) ? $rev["publisher"] : "";
                $review->pubDate = isset($rev["pubDate"]) ? $rev["pubDate"] : "";
                $review->description = isset($rev["description"]) ? $rev["description"]  : "";
                $review->lang = isset($rev["lang"]) ? $rev["lang"]  : "";
                $review->rating = isset($rev["rating"]) ? $rev["rating"]  : array();
                $review->standalone = false;

                array_push( $data , $review->toArray() );
            }
        }
        return $data;
    }
    public function toString(){
        $data = $this->toArray();
        return json_encode($data,$this->formated);
    }

    public function toArray(){
        $data = array();
        $data["@context"] = $this->context;
        $data["@type"]  = $this->type;
        $data["name"] = $this->eventName;
        $data["startDate"] = $this->startDate;
        $data["endDate"] = $this->endDate;

        $location = $this->printLocation();
        if(count($location) > 0)
            $data["location"] = $location;

        if(count($this->image) > 0)
            $data["image"] = $this->image;
        
        $data["description"] = htmlentities($this->description,ENT_QUOTES,"UTF-8");

        $offers = $this->printOffers();
        if(count($offers) > 0)
            $data["offers"] = $offers;
        
        if($this->performer != ""){
            $data["performer"] = array(
                "@type" => $this->performerType,
                "name" => $this->performer
            );
        }
        if(count($this->ratings) > 0){
            $data["AggregateRating"] = $this->printRatings();
        }

        if(count($this->reviews) > 0){
            $data["review"] = $this->printReviews();
        }

        return $data;
    }
}