<?php namespace Helper\MetadataGenerator\StructuredData\Organization;

use Helper\MetadataGenerator\StructuredData\JsonLDGenerator;
use Helper\MetadataGenerator\StructuredData\ContactType;
use Helper\MetadataGenerator\StructuredData\Review\ReviewJsonLD;
use Helper\MetadataGenerator\StructuredData\Rating\AggregateRatingsJsonLD;

class OrganizationJsonLD extends JsonLDGenerator{

    public $url = "";
    
    public $social = array();
    
    public $logo = "";
    
    public $name = "";
    
    public $contacts = array(); // [phone , email , type]
    
    public $address = array();

    public $ratings = array();
    
    public $reviews = array();

    public $formated = 0;

    public function __construct($format = false){
      parent::__construct();
      $this->type = "Organization";
      $this->formated = $format ? 128 : 0;
    }

    public function toString(){
        $data = $this->toArray();
        return json_encode($data,$this->formated);
    }
    private function printContact($contact){
        if(is_array($contact) && count($contact)>0){

            $type = ContactType::isValidValue($contact["type"], $strict = true) ? $contact["type"] : "";
            if($type != ""){
                $c = array();
                $c["@type"] = "ContactPoint";
                $c["contactType"] = $type;
                $c["telephone"] = $contact["phone"];
                if( isset($contact["email"]) )
                    $c["email"] = $contact["email"];

                return $c;
            }

        }

        return null;
    }
    private function printContacts(){
        $contacts = array();
        
        if(isset($this->contacts) &&  count($this->contacts) > 0){

            foreach($this->contacts as $contact){
                $c = $this->printContact($contact);
                if($c != null){
                    array_push($contacts,$c);
                }
            }
        }
        return $contacts;
    }

    private function printAddress(){
        $data = array();
        if(count($this->address) > 0){
            $data["@type"] = "PostalAddress";
            $data["streetAddress"] = isset($this->address["streetAddress"]) ? $this->address["streetAddress"] : "";
            $data["addressLocality"] = isset($this->address["city"]) ? $this->address["city"] : "";
            $data["addressRegion"] = isset($this->address["region"]) ? $this->address["region"] : "";
            $data["postalCode"] = isset($this->address["zipcode"]) ? $this->address["zipcode"] : "";
            $data["addressCountry"] = isset($this->address["country"]) ? $this->address["country"] : "";
        }

        return $data;
    }

    private function printRatings(){
        $ratings = new AggregateRatingsJsonLD();
        $ratings->value = isset($this->ratings["value"]) ? $this->ratings["value"] : 0;
        $ratings->maxValue  = isset($this->ratings["maxValue"]) ? $this->ratings["maxValue"] : 0;
        $ratings->minValue  = isset($this->ratings["minValue"]) ? $this->ratings["minValue"] : 0;
        $ratings->totalRatings  = isset($this->ratings["totalRatings"]) ? $this->ratings["totalRatings"] : 0;

        return $ratings->toArray();

    }

    private function printReviews(){
        $data = array();
        if(isset($this->reviews)){
            foreach($this->reviews as $rev){
                $review = new ReviewJsonLD();
                $review->url = isset($rev["url"]) ? $rev["url"] : "";
                $review->authors = isset($rev["authors"]) ? $rev["authors"] : array();
                $review->publisher = isset($rev["publisher"]) ? $rev["publisher"] : "";
                $review->pubDate = isset($rev["pubDate"]) ? $rev["pubDate"] : "";
                $review->description = isset($rev["description"]) ? $rev["description"]  : "";
                $review->lang = isset($rev["lang"]) ? $rev["lang"]  : "";
                $review->rating = isset($rev["rating"]) ? $rev["rating"]  : array();
                $review->standalone = false;

                array_push( $data , $review->toArray() );
            }
        }
        return $data;
    }
    public function toArray(){
        $data = array();
        $data["@context"] = $this->context;
        $data["@type"]  = $this->type;
        $data["url"] = $this->url;
        $data["name"] = htmlentities($this->name,ENT_QUOTES,"UTF-8");
        $data["logo"] = $this->logo;
        if(count($this->address) > 0){
            $data["address"] = $this->printAddress();
        }
        $contacts = $this->printContacts($this->contacts);
        if(count($contacts) > 0){
            $data["contactPoint"] = $contacts;
        }
        if(count($this->social) > 0){
            $data["sameAs"] = array_values($this->social);
        }
        if(count($this->ratings) > 0){
            $data["AggregateRating"] = $this->printRatings();
        }

        if(count($this->reviews) > 0){
            $data["review"] = $this->printReviews();
        }
        return $data;
    }


    /*
    contactType
    "customer support", 
    "technical support", 
    "billing support", 
    "bill payment", 
    "sales", 
    "reservations", 
    "credit card support", 
    "emergency", 
    "baggage tracking", 
    "roadside assistance", 
    "package tracking"
    */
}