<?php namespace Helper\MetadataGenerator\StructuredData\Review;

use Helper\MetadataGenerator\StructuredData\JsonLDGenerator;
use Helper\MetadataGenerator\StructuredData\LocalBusiness\LocalBusinessJsonLD;
use Helper\MetadataGenerator\StructuredData\Event\EventJsonLD;
use Helper\MetadataGenerator\StructuredData\Product\ProductJsonLD;
use Helper\MetadataGenerator\StructuredData\Organization\OrganizationJsonLD;

class ReviewJsonLD extends JsonLDGenerator{

    public $url = "";

    public $authors = array();

    public $publisher = "";

    public $pubDate = "";

    public $description = "";

    public $lang = "";

    public $rating = array();
    
    public $standalone = false;

    public $itemReviewedType = ""; //Event,Product,LocalBusiness,Organization

    public $itemReviewed = null;

    public $formated = 0;
    
    public function __construct($format = false){
        parent::__construct();
        $this->type = "Review";
        $this->formated = $format ? 128 : 0;
    }

    private function printPublisher(){
        return array(
            "@type" => "Organization", //Organization or Person
            "name" => $this->publisher
        );
    }
  
    private function printAuthor($author){
        if( is_array($author) ){
            $name = isset($author["name"]) ? htmlentities($author["name"],ENT_QUOTES,"UTF-8") : "";
            $social = array();
            foreach($author as $key=>$value){
                if($key != "name") array_push($social, $value);
            }
        }else{
            $name = $author;
            $social = array();
        }

        return array(
            "@type" => "Person",
            "name" => $name,
            "sameAs" => $social
        );
    }

    private function printAuthors(){
        $arr = array();
        if(count($this->authors) > 0){
        foreach($this->authors as $author){
            array_push($arr,$this->printAuthor($author));
        }
        }
        return $arr;
    }

    private function printRating(){
        return array(
            "@type" => "Rating",
            "worstRating" => isset($this->rating["minValue"]) ?  $this->rating["minValue"] : 0,
            "bestRating" => isset($this->rating["maxValue"]) ?  $this->rating["maxValue"] : 0,
            "ratingValue" => isset($this->rating["value"]) ?  $this->rating["value"] : 0,
        );
    }
    
    private function isValidItemToReview(){
        return strtolower( $this->itemReviewedType ) == "event" ||  
               strtolower( $this->itemReviewedType ) == "product" ||
               strtolower( $this->itemReviewedType ) == "organization" ||
               strtolower( $this->itemReviewedType ) == "localbusiness";  
    }

    private function printItemReviewed(){
        $data = array();
        if(strtolower( $this->itemReviewedType ) == "event"){

            $event = new EventJsonLD();
            $event->eventName =  isset($this->itemReviewed["eventName"]) ? $this->itemReviewed["eventName"] : "";
            $event->startDate = isset($this->itemReviewed["startDate"]) ? $this->itemReviewed["startDate"] : "";
            $event->address = isset($this->itemReviewed["address"]) ? $this->itemReviewed["address"] : array();
            $event->placeName = isset($this->itemReviewed["placeName"]) ? $this->itemReviewed["placeName"] : "";
            $event->image =  isset($this->itemReviewed["image"]) ? $this->itemReviewed["image"] : array();
            $event->description =  isset($this->itemReviewed["description"]) ? $this->itemReviewed["description"] : "";
            $event->endDate =  isset($this->itemReviewed["endDate"]) ? $this->itemReviewed["endDate"] : "";
            $event->performer =  isset($this->itemReviewed["performer"]) ? $this->itemReviewed["performer"] : "";
            $event->offers =  isset($this->itemReviewed["offers"]) ? $this->itemReviewed["offers"] : array();
            $event->performerIsGroup = isset($this->itemReviewed["performerIsGroup"]) ? $this->itemReviewed["performerIsGroup"] : false;
            $event->ratings = isset($this->itemReviewed["ratings"]) ? $this->itemReviewed["ratings"] : array();
            $event->reviews = isset($this->itemReviewed["reviews"]) ? $this->itemReviewed["reviews"] : array();
            return $event->toArray();

        }
        if(strtolower( $this->itemReviewedType ) == "product"){

            $product = new ProductJsonLD();
            $product->name = isset($this->itemReviewed["name"]) ? $this->itemReviewed["name"]  : "";
            $product->image = isset($this->itemReviewed["image"]) ? $this->itemReviewed["image"]  : array();
            $product->description = isset($this->itemReviewed["description"]) ? $this->itemReviewed["description"]  : "";
            $product->brandName = isset($this->itemReviewed["brandName"]) ? $this->itemReviewed["brandName"]  : "";
            $product->offers = isset($this->itemReviewed["offers"]) ? $this->itemReviewed["offers"] : array();
            $product->sku  = isset($this->itemReviewed["sku"]) ? $this->itemReviewed["sku"]  : "";
            $product->ratings = isset($this->itemReviewed["ratings"]) ? $this->itemReviewed["ratings"] : array();
            $product->reviews = isset($this->itemReviewed["reviews"]) ? $this->itemReviewed["reviews"] : array();
            return $product->toArray();

        }

        if(strtolower( $this->itemReviewedType ) == "localbusiness"){
           
            $business = new LocalBusinessJsonLD($this->itemReviewed["type"],$format);
            $business->url = isset($this->itemReviewed["url"]) ? $this->itemReviewed["url"] : "";
            $business->name = isset($this->itemReviewed["name"]) ? $this->itemReviewed["name"] : "";
            $business->description = isset($this->itemReviewed["description"]) ? $this->itemReviewed["description"] : "";
            $business->image = isset($this->itemReviewed["image"]) ? $this->itemReviewed["image"] : array();
            $business->phone = isset($this->itemReviewed["phone"]) ? $this->itemReviewed["phone"] : "";
            $business->email = isset($this->itemReviewed["email"]) ? $this->itemReviewed["email"] : "";
            $business->address = isset($this->itemReviewed["address"]) ? $this->itemReviewed["address"] : array();
            $business->geo  = isset($this->itemReviewed["geo"]) ? $this->itemReviewed["geo"] : array();
            $business->schedule = isset($this->itemReviewed["schedule"]) ? $this->itemReviewed["schedule"] : array();
            $business->menuUrl = isset($this->itemReviewed["menu"]) ? $this->itemReviewed["menu"] : "";
            $business->acceptsReservations = isset($this->itemReviewed["reservations"]) ? $this->itemReviewed["reservations"] : false;
            $business->servesCuisine = isset($this->itemReviewed["cuisine"]) ? $this->itemReviewed["cuisine"] : "";
            $business->social = isset($this->itemReviewed["social"]) ? $this->itemReviewed["social"] : array();
            $business->priceRange = isset($this->itemReviewed["priceRange"]) ? $this->itemReviewed["priceRange"] : "";
            $business->map = isset($this->itemReviewed["map"]) ? $this->itemReviewed["map"] : "";
            $business->departments = isset($this->itemReviewed["departments"]) ? $this->itemReviewed["departments"] : array();
            $business->ratings = isset($this->itemReviewed["ratings"]) ? $this->itemReviewed["ratings"] : array();
            $business->reviews = isset($this->itemReviewed["reviews"]) ? $this->itemReviewed["reviews"] : array();
            return $business->toString(true);
        }

        if(strtolower( $this->itemReviewedType ) == "organization"){

            $organization = new OrganizationJsonLD();
            $organization->url = isset($this->itemReviewed["url"]) ? $this->itemReviewed["url"] : "";
            $organization->social = isset($this->itemReviewed["social"]) ? $this->itemReviewed["social"] : array();
            $organization->logo = isset($this->itemReviewed["logo"]) ? $this->itemReviewed["logo"] : "";
            $organization->name = isset($this->itemReviewed["name"]) ? $this->itemReviewed["name"] : "";
            $organization->contacts = isset($this->itemReviewed["contacts"]) ? $this->itemReviewed["contacts"] : array();
            $organization->address = isset($this->itemReviewed["address"]) ? $this->itemReviewed["address"] : array();
            $organization->ratings = isset($this->itemReviewed["ratings"]) ? $this->itemReviewed["ratings"] : array();
            $organization->reviews = isset($this->itemReviewed["reviews"]) ? $this->itemReviewed["reviews"] : array();
            return $organization->toArray();
        }

    }

    public function toArray(){
        $data = array();
        $data["@context"] = $this->context;
        $data["@type"]  = $this->type;
        $data["url"] = $this->url;
        $data["inLanguage"] = $this->lang;
        $data["datePublished"] = $this->pubDate;
        $data["description"] = htmlentities($this->description,ENT_QUOTES,"UTF-8");
        $data["publisher"] = $this->printPublisher();
        $data["author"] = $this->printAuthors();

        if(count($this->rating) > 0){
            $data["reviewRating"] = $this->printRating();
        }
        
        if($this->standalone && 
           $this->isValidItemToReview() && 
           isset($this->itemReviewed) && 
           count($this->itemReviewed) > 0){

            $data["itemReviewed"] = $this->printItemReviewed();

        }

        return $data;
    }

    public function toString(){
        $data = $this->toArray();
        return json_encode($data,$this->formated);
    }
}